import pytest
from rusty_results.prelude import *


def test_err_builds():
    msg: str = "Bang!"
    err:  Result[int, str] = Err(msg)
    assert err.Error == msg
    assert err.is_err
    assert not err.is_ok


def test_err_repr():
    assert Err(0) == eval(repr(Err(0)))


def test_err_contains():
    err: Result[int, int] = Err(0)
    assert not err.contains(0)
    assert err.contains_err(0)


def test_err_ok():
    err: Result[int, int] = Err(0)
    assert err.ok() == Empty()


def test_err_err():
    err: Result[int, int] = Err(0)
    assert  err.err() == Some(0)


def test_err_map():
    def function(i: int) -> int:
        return i + 10

    lambda_function: Callable[[int], int] = lambda x: x+100

    err: Result[int, int] = Err(0)

    assert err.map(function) == err
    assert err.map(lambda_function) == err


def test_err_map_or():
    def function(i: int) -> int:
        return i + 10

    lambda_function: Callable[[int], int] = lambda x: x+100

    err: Result[int, int] = Err(0)

    assert err.map_or(1, function) == 1
    assert err.map_or(1, lambda_function) == 1


def test_err_map_or_else():
    def function(i: int) -> int:
        return i + 10

    lambda_function: Callable[[int], int] = lambda x: x+100

    err: Result[int, int] = Err(0)

    assert err.map_or_else(function, lambda_function) == 10
    assert err.map_or_else(lambda_function, function) == 100


def test_err_map_err():
    def function(i: int) -> int:
        return i + 10

    lambda_function: Callable[[int], int] = lambda x: x+100

    err: Result[int, int] = Err(0)

    assert err.map_err(function) == Err(10)
    assert err.map_err(lambda_function) == Err(100)


def test_err_iter():
    err: Result[int, int] = Err(0)
    assert list(err) == []


def test_err_and_then():
    err: Result[int, int] = Err(0)

    def op(x: int) -> Result[int, int]:
        return Ok(x+10)

    assert err.and_then(op) == err


def test_err_or_else():
    err: Result[int, int] = Err(0)

    def op(e: int) -> int:
        return e+10

    assert err.or_else(op) == Err(10)


def test_err_unwrap():
    err: Result[int, int] = Err(0)
    with pytest.raises(UnwrapException):
        err.unwrap()


def test_err_unwrap_or():
    err: Result[int, int] = Err(0)
    assert err.unwrap_or(1) == 1


def test_err_unwrap_or_else():
    err: Result[int, int] = Err(0)
    assert err.unwrap_or_else(lambda: 1) == 1


def test_err_expect():
    exception_msg = "foo"
    err: Result[int, int] = Err(0)
    with pytest.raises(UnwrapException) as e:
        err.expect(exception_msg)
    assert str(e.value) == exception_msg


def test_err_unwrap_err():
    err: Result[int, int] = Err(0)
    assert err.unwrap_err() == 0


def test_err_expect_err():
    err: Result[int, int] = Err(0)
    assert err.expect_err("foo") == 0



from rusty_results.prelude import *


def test_option_and():
    assert (Some(5) and Some(10)) == Some(10)
    assert (Some(5) and Empty()) == Empty()
    assert (Empty() and Empty()) == Empty()
    assert (Empty() and Some(10)) == Empty()


def test_option_or():
    assert (Some(5) or Some(10)) == Some(5)
    assert (Some(5) or Empty()) == Some(5)
    assert (Empty() or Empty()) == Empty()
    assert (Empty() or Some(10)) == Some(10)


def test_option_hash():
    assert len({Some(0), Empty()}) == 2
    assert len({Some(0), Some(1)}) == 2
    assert len({Empty(), Empty()}) == 1
    assert len({Some(0), Some(0)}) == 1
    assert len({Empty(), Empty()}) == 1



from typing import Tuple

import pytest

from rusty_results.prelude import *


def create_some(value: int = 10) -> Tuple[Some[int], int]:
    return Some(value), value


def test_some_builds():
    some, value = create_some()
    assert some.Value == value


def test_some_is_some():
    some, value = create_some()
    assert some.is_some


def test_some_is_empty():
    some, value = create_some()
    assert not some.is_empty


def test_some_contains():
    some, value = create_some()
    assert some.contains(value)
    assert not some.contains(94)


def test_some_expects():
    some, value = create_some()
    assert some.expects("") == value
    assert some.expects("test") == value


def test_some_unwrap():
    some, value = create_some()
    assert some.unwrap() == value


def test_some_unwrap_or():
    some, value = create_some()
    assert some.unwrap_or(94) == value


def test_some_unwrap_or_else():
    some, value = create_some(0)
    assert some.unwrap_or_else(lambda x: 94) == value


def test_some_map():
    some, value = create_some()
    assert some.map(f=lambda x: x+5) == Some(value + 5)


def test_some_map_or():
    some, value = create_some()
    assert some.map_or(default=20, f=lambda x: x+5) == Some(value + 5)


def test_some_map_or_else():
    some, value = create_some()
    ret = some.map_or_else(default=lambda x: 94, f=lambda x: x+5)
    assert ret == Some(value + 5)
    assert ret.Value == value + 5


def test_some_iter():
    some, value = create_some()
    assert list(some.iter()) == [10]


def test_some_filter():
    some, value = create_some()
    assert some.filter(predicate=lambda x: True) == some
    assert some.filter(predicate=lambda x: True).Value == value
    assert some.filter(predicate=lambda x: False) == Empty()


def test_some_ok_or():
    some, value = create_some()
    assert some.ok_or(0) == Ok(10)


def test_some_ok_or_else():
    some, value = create_some()
    assert some.ok_or_else(lambda: 0) == Ok(10)


def test_some_and_then():
    some, value = create_some()
    assert some.and_then(lambda x: Some(x + 5)).Value == value + 5
    assert some.and_then(lambda x: Some(x + 5)) == Some(value + 5)


def test_some_or_else():
    some, value = create_some()
    assert some.or_else(f=lambda x: Some(x + 5)) == some


def test_some_xor():
    some, value = create_some()
    assert some.xor(optb=Empty()) == some
    assert some.xor(optb=Some(4)) == Empty()


def test_some_zip():
    some, value = create_some()
    assert some.zip(Empty()) == Empty()
    assert some.zip(Some(5)) == Some((value, 5))


def test_some_zip_with():
    some, value = create_some()
    assert some.zip_with(Empty(), lambda x, y: x + y) == Empty()
    assert some.zip_with(Some(5), lambda x, y: x + y) == Some(value + 5)


def test_some_expect_none():
    some, value = create_some()
    with pytest.raises(Exception):
        some.expect_empty("")


def test_some_unwrap_empty():
    some, value = create_some()
    with pytest.raises(Exception):
        some.expect_empty("")



import pytest
from rusty_results.prelude import *


def test_result_eq():
    assert Ok(0) == Ok(0)
    assert Err(0) == Err(0)
    assert Ok(0) != Ok(1)
    assert Err(0) != Err(1)
    assert Ok(0) != Err(0)
    assert Ok("0") != Ok(0)
    assert Ok(0) != 0


def test_result_hash():
    assert len({Ok(0), Err(0)}) == 2
    assert len({Ok(0), Ok(1)}) == 2
    assert len({Err(0), Err(1)}) == 2
    assert len({Ok(0), Ok(0)}) == 1
    assert len({Err(0), Err(0)}) == 1


def test_result_and():
    assert (Ok(0) and Ok(1)) == Ok(1)
    assert (Ok(0) and Err(0)) == Err(0)
    assert (Err(0) and Err(1)) == Err(0)
    assert (Err(0) and Ok(1)) == Err(0)


def test_result_or():
    assert (Ok(0) or Ok(1)) == Ok(0)
    assert (Ok(0) or Err(0)) == Ok(0)
    assert (Err(0) or Err(1)) == Err(1)
    assert (Err(0) or Ok(1)) == Ok(1)



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="rusty-results",
    version="0.0.1b1",
    author="Daniel Sanchez Quiros, Antonio Jose Checa Bustos",
    author_email="sanchez.quiros.daniel@gmail.com, antonio.checa.bustos@gmail.com",
    description="Rust inspired Result and Option types",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="",
    packages=setuptools.find_packages(
        ".",
        exclude=[
            "tests*",
            "examples*"
        ]
    ),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7'
)


import pytest
from rusty_results.prelude import *


def test_ok_builds():
    value: int = 10
    ok: Result[int, str] = Ok(value)
    assert ok.Value == value
    assert ok.is_ok
    assert not ok.is_err


def test_ok_repr():
    assert Ok(0) == eval(repr(Ok(0)))


def test_ok_contains():
    ok: Result[int, int] = Ok(0)
    assert ok.contains(0)
    assert not ok.contains_err(0)


def test_ok_ok():
    ok: Result[int, int] = Ok(0)
    assert ok.ok() == Some(0)


def test_ok_err():
    ok: Result[int, int] = Ok(0)
    assert ok.err() == Empty()


def test_ok_map():
    def function(i: int) -> int:
        return i + 10

    lambda_function: Callable[[int], int] = lambda x: x+100

    ok: Result[int, int] = Ok(0)

    assert ok.map(function) == Ok(10)
    assert ok.map(lambda_function) == Ok(100)


def test_ok_map_or():
    def function(i: int) -> int:
        return i + 10

    lambda_function: Callable[[int], int] = lambda x: x+100

    ok: Result[int, int] = Ok(0)

    assert ok.map_or(1, function) == 10
    assert ok.map_or(1, lambda_function) == 100


def test_ok_map_or_else():
    def function(i: int) -> int:
        return i + 10

    lambda_function: Callable[[int], int] = lambda x: x+100

    ok: Result[int, int] = Ok(0)

    assert ok.map_or_else(lambda_function, function) == 10
    assert ok.map_or_else(function, lambda_function) == 100


def test_ok_map_err():
    def function(i: int) -> int:
        return i + 10

    lambda_function: Callable[[int], int] = lambda x: x+100

    ok: Result[int, int] = Ok(0)

    assert ok.map_err(function) == ok
    assert ok.map_err(lambda_function) == ok


def test_ok_iter():
    ok: Result[int, int] = Ok(0)
    assert list(ok) == [0]


def test_ok_and_then():
    ok: Result[int, int] = Ok(0)

    def op(x: int) -> Result[int, int]:
        return Ok(x+10)

    assert ok.and_then(op) == Ok(10)


def test_ok_or_else():
    ok: Result[int, int] = Ok(0)

    def op(e: int) -> int:
        return e+10

    assert ok.or_else(op) == ok


def test_ok_unwrap():
    ok: Result[int, int] = Ok(0)
    assert ok.unwrap() == 0


def test_ok_unwrap_or():
    ok: Result[int, int] = Ok(0)
    assert ok.unwrap_or(1) == 0


def test_ok_unwrap_or_else():
    ok: Result[int, int] = Ok(0)
    assert ok.unwrap_or_else(lambda: 1) == 0


def test_ok_expect():
    ok: Result[int, int] = Ok(0)
    assert ok.expect("foo") == 0


def test_ok_unwrap_err():
    ok: Result[int, int] = Ok(0)
    with pytest.raises(UnwrapException):
        ok.unwrap_err()


def test_ok_expect_err():
    exception_msg = "foo"
    ok: Result[int, int] = Ok(0)
    with pytest.raises(UnwrapException) as e:
        ok.expect_err(exception_msg)
    assert str(e.value) == exception_msg



import pytest

from rusty_results.prelude import *


def test_empty_is_some():
    assert not Empty().is_some


def test_empty_is_empty():
    assert Empty().is_empty


def test_empty_contains():
    assert not Empty().contains(93)


def test_empty_expects():
    with pytest.raises(Exception):
        Empty().expects("")


def test_empty_unwrap():
    with pytest.raises(Exception):
        Empty().expects()


def test_empty_unwrap_or():
    assert Empty().unwrap_or(30) == 30


def test_empty_unwrap_or_else():
    assert Empty().unwrap_or_else(lambda: 50) == 50


def test_empty_map():
    assert Empty().map(lambda x: 40) == Empty()


def test_empty_map_or():
    assert Empty().map_or(40, lambda x: x + 4) == Some(40)


def test_empty_map_or_else():
    assert Empty().map_or_else(lambda: 40, lambda x: x+3) == Some(40)


def test_empty_iter():
    assert list(Empty().iter()) == list(iter([]))


def test_empty_filter():
    assert Empty().filter(lambda x: True) == Empty()
    assert Empty().filter(lambda x: False) == Empty()


def test_empty_ok_or():
    assert Empty().ok_or(0) == Err(0)


def test_empty_ok_or_else():
    assert Empty().ok_or_else(lambda: 0) == Err(0)


def test_empty_and_then():
    assert Empty().and_then(lambda x: Some(4)) == Empty()
    assert Empty().and_then(lambda x: 0) == Empty()


def test_empty_or_else():
    assert Empty().or_else(lambda: Some(4)) == Some(4)


def test_empty_xor():
    assert Empty().xor(Some(5)) == Some(5)
    assert Empty().xor(Empty()) == Empty()


def test_empty_zip():
    assert Empty().zip(40) == Empty()


def test_empty_zip_with():
    assert Empty().zip_with(Some(4), lambda x, y: x + y) == Empty()


def test_empty_expect_none():
    Empty().expect_empty("")


def test_empty_unwrap_empty():
    Empty().unwrap_empty()
